@@ -1,6 +1,6 @@
 
 /*
- *	$Id: gui.c,v 1.20 1997/05/14 12:52:47 mark Exp $ 
+ *	$Id: gui.c,v 1.27 1997/05/19 09:17:44 mark Exp $ 
  *
  *	Gui.c : This file well contain most of the user-interface code, 
  *	allowing compilation on different operating systems and other 
@@ -56,7 +56,31 @@
 
 /* Maximum number of remailers to use */
 
-#define MAXMIX	6
+#ifndef MAXMIX
+#define MAXMIX	4
+#endif
+
+/* Minimum number of remailers to use */
+
+#ifndef MINMIX
+#define MINMIX  3
+#endif
+
+/* Minimum reliabilty * 100 */
+
+#ifndef MINMIXREL
+#define MINMIXREL	7500
+#endif
+
+/* Minimum path reliability * 100 */
+
+#ifndef MINMIXPATHREL
+#define MINMIXPATHREL	5000
+#endif
+
+/* Maximum number of tries to find a reliable path */
+
+#define MAX_MIX_TRIES	4
 
 /* Local variables */
 
@@ -114,6 +138,8 @@ static	char	*remail_args[32] =
 	"-f",
 	"-to"
 };
+
+static	char	mix_path[] = MIXPATH;
 #endif
 
 #ifndef NO_PREMAIL
@@ -147,6 +173,7 @@ void	(*callback_proc)();
 COMPOSE_WINDOW	*callback_win;
 
 extern	char	*our_userid;
+extern char	*current_nym(void);
 
 #ifdef PGPTOOLS
 static	byte	md5_pass[16];
@@ -161,6 +188,8 @@ int	layout_compact = TRUE;
 int	layout_compact = FALSE;
 #endif
 
+extern	char	*current_nym (void);
+
 /* Program name */
 
 #ifdef XSAFEMAIL
@@ -169,7 +198,7 @@ char	prog_name[] = "XSafeMail";
 char	prog_name[] = "Privtool";
 #endif
 
-char	prog_ver [] = "V0.89 BETA";
+char	prog_ver [] = "V0.90 BETA";
 
 /* Set the description of the message for the message list */
 
@@ -378,6 +407,9 @@ MESSAGE	*m;
 	int	i;
 	DISPLAY_WINDOW	*w;
 
+	if(m == NULL)
+	    return;
+
 	/* Add time to random seed */
 
 	update_random();
@@ -617,12 +649,19 @@ MESSAGE	*m;
 				free_buffer (b);
 				free_buffer (fm);
 	
-				display_message(newm);
-	
-#ifndef MOTIF
 				clear_display_footer (w);
-#endif
 				clear_busy ();
+
+				/* Just for safety; don't run premail
+				   a second time as we could end up in
+				   a loop. */
+
+				m = newm;
+				if (!(newm->flags & MESS_NYM))
+					display_message(newm);
+				else
+					goto display_plaintext;
+	
 				return;
 			}
 			else {
@@ -1016,10 +1055,7 @@ void	decrypt_with_passphrase()
 	sync_list();
 }
 
-void	set_flags_from_decryption(m,i)
-
-MESSAGE	*m;
-int	i;
+void	set_flags_from_decryption(MESSAGE *m, int i)
 
 {
 	m->flags |= MESS_VERIFIED;
@@ -1062,7 +1098,7 @@ int	i;
 static	char	*replying_message_id;
 static	char	*replying_message_sender;
 
-COMPOSE_WINDOW	*setup_send_window()
+COMPOSE_WINDOW	*setup_send_window(void)
 
 {
 	if (replying_message_id) {
@@ -1080,9 +1116,7 @@ COMPOSE_WINDOW	*setup_send_window()
 
 /* Set up reply variables from the message we're replying to */
 
-void	set_reply (m)
-
-MESSAGE	*m;
+void	set_reply (MESSAGE *m)
 
 {
 	if (replying_message_id) {
@@ -1103,9 +1137,7 @@ MESSAGE	*m;
 	}
 }
 
-static	int	remove_duplicates (list)
-
-char	**list;
+static	int	remove_duplicates (char **list)
 
 {
 	int	i, j;
@@ -1147,10 +1179,7 @@ char	**list;
 	return j;
 }
 
-static	add_list_to_header (b, l)
-
-BUFFER	*b;
-char	**l;
+static	add_list_to_header (BUFFER *b, char **l)
 
 {
 	int	i;
@@ -1172,9 +1201,7 @@ char	**l;
 
 /* Support folder specification */
 
-char	*expand_filename (s)
-
-char	*s;
+char	*expand_filename (char *s)
 
 {
 	char	*folder;
@@ -1249,21 +1276,27 @@ char	*s;
 }
 
 #ifndef NO_MIXMASTER
-static	setup_remail_args(addrs, subject, temp)
-
-char	*addrs, *subject;
-char	*temp;
+static	setup_remail_args(char *addrs, char *subject, char *temp)
 
 {
-	int	i, j;
+	int	i, j, a, a2;
 	int	l;
-	int	c;
+	int	c, cnt;
+	int	mix;
 	FILE	*mix_fp;
-	char	mix_path[1024];
+	int	tot_reliable;
+	int	path_reliable = 0;
+	char	file_path[1024];
+	char	line[1024];
 	static	char	n[MAXMIX][5];
 	char	used[100];
+	char	name[100][64];
+	int	reliable[100];
+	char	found_mix = FALSE;
+	char	reading_mix = FALSE;
+	char	*p, *q;
 
-	i = 5;
+	a = 5;
 
 #ifndef MIXMASTER_STDIN
 	remail_args[2] = temp;
@@ -1271,29 +1304,100 @@ char	*temp;
 
 	/* Pass subject if neccesary */
 
-	remail_args[i++] = addrs;
+	remail_args[a++] = addrs;
 
-	remail_args[i++] = "-s";
+	remail_args[a++] = "-s";
 	if (subject && *subject) 
-		remail_args[i++] = subject;
+		remail_args[a++] = subject;
 	else
-		remail_args[i++] = "None";
+		remail_args[a++] = "None";
 
 	/* Open type2.list */
 
-	sprintf(mix_path, "%s/type2.list", MIXPATH);
+	sprintf(file_path, "%s/type2.list", mix_path);
 
-	mix_fp = fopen(mix_path, "rt");
+	mix_fp = fopen(file_path, "rt");
 	if (!mix_fp)
 		return FALSE;
 
-	/* Count number of lines in type2.list */
+	/* Count number of lines in type2.list and copy names over */
 
 	l = 0;
-	while (!feof(mix_fp)) {
-		c = fgetc(mix_fp);
-		if (c == '\n')
-			l++;
+	while (!feof(mix_fp) && l < 100) {
+		p = name[l];
+
+		do {
+			*p++ = c = fgetc(mix_fp);
+		} while (c != EOF && c != '\n' && c != ' ');
+		p[-1] = 0;
+
+		while (c != '\n' && c!= EOF)
+			c = fgetc (mix_fp);
+
+		reliable[l++] = 9500;
+	}
+
+	fclose (mix_fp);
+
+	sprintf(file_path, "%s/reliability", mix_path);
+	mix_fp = fopen(file_path, "rt");
+	if (!mix_fp)
+		return FALSE;
+
+	/* Process the reliability data. First look for mixmaster
+	   followed by long line of dashes. */
+
+	while (!feof (mix_fp) && l < 100) {
+		p = line;
+		do {
+			*p++ = c = fgetc (mix_fp);
+		} while (c != EOF && c != '\n');
+		p [-1] = 0;
+
+		if (!found_mix) {
+			if (!strncmp (line, "mixmaster", 9)) {
+				found_mix = TRUE;
+			}
+		}
+		else if (!reading_mix) {
+			p = line;
+			while (*p == '-') p++;
+			if (*p)
+				found_mix = FALSE;
+			else
+				reading_mix = TRUE;
+		}
+		else {
+			int	r = 0;
+
+			p = line;
+			if (*p == '<' || !*p)
+				break;
+
+			for (i = 0; i < l; i++) {
+
+				if (!strncasecmp (line, name[i], 
+					strlen (name[i]))) {
+
+					/* Column starts around character
+					   37 */
+
+					p = line + 37;
+					while (isspace (*p)) p++;
+
+					while (*p && *p != '%') {
+						if (*p != '.') {
+							r *= 10;
+							r += *p - '0';
+						}
+						p++;
+					}
+
+					reliable[i] = r;
+					break;
+				}
+			}
+		}
 	}
 
 	fclose (mix_fp);
@@ -1301,40 +1405,76 @@ char	*temp;
 	/* Ok, we now know how many remailers are available. We only
 	   select from first 100 */
 
-	if (!l)
-		return FALSE;
+	a2 = a;
 
-	if (l > 100)
-		l = 100;
+	for (cnt = 0; cnt < MAX_MIX_TRIES && path_reliable < MINMIXPATHREL; 
+		cnt++) {
 
-	for (c = 0; c < l; c++)
-	 	used[c] = FALSE;
+		path_reliable = 10000;
+		tot_reliable = 0;
+		i = 0;
+		a = a2;
 
-	c = l;
-	if (c > MAXMIX)
-		c = MAXMIX;
+		for (c = 0; c < l; c++) {
+			if (reliable[c] > MINMIXREL) {
+		 		used[c] = FALSE;
+				tot_reliable += reliable [c];
+				i++;
+			}
+			else
+				used[c] = TRUE;
+		}
+	
+		/* If too few remailers with reliability above 75%, abort */
 
-	remail_args[i++] = "-l";
+		if (i <= MINMIX)
+			return FALSE;
 
-	/* This isn't absolutely secure, but it's not *that* important */
+		c = (i - 1);
+		if (c > MAXMIX)
+			c = MAXMIX;
+
+		remail_args[a++] = "-l";
+
+		/* This isn't absolutely secure, but it's not *that* 
+		   important */
 
-	do {
 		do {
+
+			/* We basically scale the probability of choosing a 
+			   remailer by the reliability. */
+
 #ifdef PGPTOOLS
-			j = our_randombyte() * 256 + our_randombyte();
+			j = our_randombyte() * 65536 + 
+				our_randombyte() * 256 + our_randombyte();
 #else
-			j = random();
+			j = (random() & 0xFF) * 65536 + random();
 #endif
-			j %= l;
-		} while (used[j]);
+			j %= tot_reliable;
 
-		used[j] = TRUE;
-		sprintf(n[c], "%d", j + 1);
+			for (i = 0; i < l && j > 0; i++) {
+				if (!used[i]) {
+					j -= reliable[i];
+				}
+			}
 
-		remail_args[i++] = n[c];
-	} while (--c);
+			/* Update the path reliability */
 
-	return TRUE;
+			path_reliable *= reliable [i-1];
+			path_reliable /= 10000;
+
+			tot_reliable -= reliable[i-1];
+			used[i-1] = TRUE;
+			sprintf(n[c], "%d", i);
+
+			remail_args[a++] = n[c];
+		} while (--c);
+	}
+
+	if (path_reliable >= MINMIXPATHREL) 
+		return TRUE;
+
+	return FALSE;
 }
 #endif
 
@@ -1728,7 +1868,7 @@ try_again:
 			char	temp[1024];
 			FILE	*mix_fp;
 
-			sprintf(temp, "%s/temp.mix", MIXPATH);
+			sprintf(temp, "%s/temp.mix", mix_path);
 			mix_fp = fopen(temp, "wt");
 
 			if (!mix_fp)
@@ -1786,10 +1926,15 @@ remail_error_exit:
 
 			i = 0;
 			while (cc_addrs && cc_addrs[i]) {
-				if (setup_remail_args(cc_addrs[i], subject))
+				if (setup_remail_args(cc_addrs[i], subject,
+					temp))
 					run_program(remail_args[0],
 						mail_message->message,
+#ifndef MIXMASTER_STDIN
 						mail_message->length,
+#else
+						0,
+#endif
 						remail_args,
 						NULL, NULL);
 				else 
@@ -1799,10 +1944,15 @@ remail_error_exit:
 
 			i = 0;
 			while (bcc_addrs && bcc_addrs[i]) {
-				if (setup_remail_args(bcc_addrs[i], subject))
+				if (setup_remail_args(bcc_addrs[i], subject,
+					temp))
 					run_program(remail_args[0],
 					mail_message->message,
+#ifndef MIXMASTER_STDIN
 					mail_message->length,
+#else
+					0,
+#endif
 					remail_args,
 					NULL, NULL);
 				else
