@@ -1,6 +1,6 @@
 
 /*
- *	$Id: mail_reader.c,v 1.10 1997/05/14 12:48:54 mark Exp $ 
+ *	$Id: mail_reader.c,v 1.11 1997/05/19 09:17:44 mark Exp $ 
  *
  *	(c) Copyright 1993-1997 by Mark Grant, and by other
  *	authors as appropriate. All right reserved.
@@ -102,7 +102,7 @@ static	void	set_mail (void)
 #endif
 }
 
-#ifndef linux
+#if !defined(linux) && !defined(__sgi)
 #ifdef SYSV
 #ifdef NON_ANSI
 void	usleep(n)
