@@ -51,8 +51,3 @@ typedef struct {
 
 } LIST;
 
-extern	char	*nym_name();
-extern	char	*current_nym();
-extern  char    *template_name();
-extern  char    *search_templatename();
-extern	void	clear_list();
