@@ -1,6 +1,6 @@
 /*
  *	@(#)motif.c	1.0  13/10/96
- *	$Id: motif.c,v 1.1 1997/05/10 11:10:01 mark Exp $
+ *	$Id: motif.c,v 1.2 1997/05/19 09:17:44 mark Exp $
  *
  *	(c) Copyright 1993-1996 by Mark Grant, and by other
  *	authors as appropriate. All right reserved.
@@ -81,6 +81,7 @@
 #include	"folderwin.xpm"
 
 extern char		*our_userid;
+extern char		*current_nym(void);
 
 void			update_message_list(void);
 void			set_foldwin_toggles(Boolean);
@@ -1236,8 +1237,8 @@ compose_find_free()
 	win->deliver_flags &= ~DELIVER_REMAIL;
 
     /* Check mailrc for logging options */
-    log = find_mailrc("nodontlogmessages");
-    dontlog = find_mailrc("dontlogmessages");
+    log = (find_mailrc("nodontlogmessages") != NULL);
+    dontlog = (find_mailrc("dontlogmessages") != NULL);
 
     /* If logging enabled, set log options */
     if (log && !dontlog) {
