@@ -1,6 +1,6 @@
 
 /*
- *	$Id: pgplib.c,v 1.24 1997/04/27 19:18:11 mark Exp $ 
+ *	$Id: pgplib.c,v 1.26 1997/05/19 09:17:44 mark Exp $ 
  *
  *	pgplib.c, (mostly) by Mark Grant 1993-1997
  *
@@ -44,6 +44,7 @@
 #include <stdlib.h>
 #else
 #include <malloc.h>
+#include <limits.h>
 #endif
 #include <sys/types.h>
 #include <unistd.h>
@@ -88,6 +89,10 @@
 
 extern	char	*getenv();
 
+#ifdef __sgi
+#define vfork fork
+#endif
+
 /* BUF_SIZE is the general size of stack buffers. Should this ever be used 
    on DOS, you might want to reduce the value of BUF_SIZE, or replace the 
    stack use with a malloc(). */
@@ -1601,7 +1606,6 @@ int	run_program(char *prog, byte *message, int msg_len,
 		pipe (pass_fd);
 
 	/* Fork and execute the program */
-
 	if (!(child_pid = vfork())) {
 		char	pass_env[32];
 
@@ -1982,9 +1986,25 @@ int	decrypt_message (BUFFER *message, BUFFER *decrypted, BUFFER *signature,
 			if (!get_public_key (key, userid, &public, &trust)) {
 
 				/* If we didn't get it, then copy the message
-				   over. */
+				   over after converting the dashes. */
 
-				add_to_buffer (decrypted, s, e - s);
+				if (!strncmp("- ", s, 2))
+					s+= 2;
+
+				t = s;
+
+				while (t < e) {
+					m = (byte *)strstr (t, "\n- ");
+
+					if (!m || m > e) {
+						add_to_buffer (decrypted,
+							t, (e - t));
+						break;
+					}
+					add_to_buffer (decrypted, 
+						t, (m - t) + 1);
+					t = m + 3;
+				}
 
 				/* We return an error message in the signature
 				   buffer, giving the id of the key in hex */
