@@ -1,5 +1,5 @@
 /*
- *	$Id: x.c,v 1.10 1997/05/14 10:25:32 mark Exp $ 
+ *	$Id: x.c,v 1.14 1997/05/16 15:59:25 mark Exp $ 
  *
  *	(c) Copyright 1993-1997 by Mark Grant, and by other
  *	authors as appropriate. All right reserved.
@@ -762,7 +762,6 @@ void	remail_failed_notice_proc(void)
 			"Remail Failed !",
 			NULL,
 		NOTICE_BUTTON_YES, "Abort",
-		NOTICE_BUTTON_NO, "Ignore",
 		NULL);
 
 	update_random ();
@@ -2742,6 +2741,7 @@ void set_initial_scrollbar_position(void)
 	int p = 1;
 	MESSAGE *m = messages.start;
 	int	v;
+	int	found_new = FALSE;
 
 	v = xv_get(v_scroll, SCROLLBAR_VIEW_LENGTH);
 
@@ -2751,17 +2751,33 @@ void set_initial_scrollbar_position(void)
 	xv_set(v_scroll, SCROLLBAR_VIEW_LENGTH, 1, 0);
 	if (messages.number > v) {
 		p = messages.number - v + 3 /* the +3 is due to error in value returned for SCROLLBAR_VIEW_LENGTH) */;
-		xv_set (v_scroll, SCROLLBAR_VIEW_START, p-1, 0);
 	}
 
+	/* Look for first new message */
+
 	while (m) {
-		if (m->status == MSTAT_NONE)
-			if (m->list_pos < p) {
-				p = m->list_pos;
-				xv_set(v_scroll, SCROLLBAR_VIEW_START, p-1, 0);
-			}
+		if (m->status == MSTAT_NONE) {
+			p = m->list_pos;
+			found_new = TRUE;
+			break;
+		}
 		m = m->next;
 	}
+
+	/* Ok, now look for first unread if we didn't find a new one */
+
+	if (!found_new) {
+		m = messages.start;
+		while (m) {
+			if (m->status == MSTAT_UNREAD) {
+				p = m->list_pos;
+				break;	
+			}
+			m = m->next;
+		}
+	}
+
+	xv_set (v_scroll, SCROLLBAR_VIEW_START, p-1, 0);
 	xv_set(v_scroll, SCROLLBAR_VIEW_LENGTH, v, 0);
 }
 
@@ -3509,22 +3525,6 @@ PANEL_LABEL_STRING, "Mail File:",
 		PANEL_NOTIFY_PROC, delete_message_proc,
 		NULL);
 
-	button_menu = (Menu) xv_create(XV_NULL, MENU,
-		MENU_ITEM,
-			MENU_STRING, "Print Decrypted/Authenticated",
-			MENU_NOTIFY_PROC, print_cooked_proc,
-			NULL,
-		MENU_ITEM,
-			MENU_STRING, "Print Encrypted/Clearsigned",
-			MENU_NOTIFY_PROC, print_raw_proc,
-			NULL,
-		NULL);
-
-	(void) xv_create (top_panel, PANEL_BUTTON,
-		PANEL_LABEL_STRING, "Print",
-		PANEL_ITEM_MENU, button_menu,
-		NULL);
-
 	button_menu = (Menu) xv_create(XV_NULL, MENU,
 		MENU_ITEM,
 			MENU_STRING, "To Sender",
@@ -3616,13 +3616,35 @@ PANEL_LABEL_STRING, "Mail File:",
 		PANEL_ITEM_MENU, files_menu[2],
 		NULL);
 
+	button_menu = (Menu) xv_create(XV_NULL, MENU,
+		MENU_ITEM,
+			MENU_STRING, "Print Decrypted/Authenticated",
+			MENU_NOTIFY_PROC, print_cooked_proc,
+			NULL,
+		MENU_ITEM,
+			MENU_STRING, "Print Encrypted/Clearsigned",
+			MENU_NOTIFY_PROC, print_raw_proc,
+			NULL,
+		NULL);
+
+	(void) xv_create (top_panel, PANEL_BUTTON,
+		PANEL_LABEL_STRING, "Print",
+		PANEL_ITEM_MENU, button_menu,
+		NULL);
+
 	(void) xv_create (top_panel, PANEL_BUTTON,
 		PANEL_LABEL_STRING, "Props",
 		PANEL_ITEM_MENU,
-			xv_create(XV_NULL, MENU, MENU_ITEM,
+			xv_create(XV_NULL, MENU, 
+				MENU_ITEM,
+				MENU_STRING, "Edit",
+				MENU_NOTIFY_PROC, properties_proc,
+				NULL,
+				MENU_ITEM,
 				MENU_STRING, "Save layout",
 				MENU_NOTIFY_PROC, save_xresources,
-				NULL, NULL),
+				NULL,
+				NULL),
 		NULL);
 
 	button_menu = (Menu) xv_create(XV_NULL, MENU, NULL);
